
-- Input file to alex to generate lexer for arithmetical expressions.

-- An alex file consists of 6 parts.
-- Haskell code is contained in { braces }.

-- 1. Haskell prologue: module header and imports.

{
module CalcLexer where
}

-- 2. Wrapper selection to generate wrapper code for the lexer.
-- Pick any of
--
-- * "basic": creates a Haskell function
--     alexScanTokens :: String -> [Token]
--
-- * "posn": to add position information to the tokens
--
-- * "monad": for a monadic lexer

%wrapper "basic"

-- 3. Macros defining regular expressions:
--
-- $id : character set macros      -- e.g.: 0-9
-- @id : regular expression macros -- e.g.: @a | $b ($c *)

$digit  = 0-9
$digit1 = 1-9
@num    = 0 | $digit1 ($digit *)

-- 4. Magic separator.
-- The name "calcTokens" we give to our lexer seems to be ignored.

calcTokens :-

-- 5. Lexing rules, taking one of the two forms
--
--   regex ;                 -- to output no token
--   regex { haskell code }  -- haskell code generates token from lexed string

$white+ ;
@num  { \ s -> Number (read s) }
"+"   { \ s -> Plus  }
"-"   { \ s -> Minus }
"*"   { \ s -> Times }
"/"   { \ s -> Div   }

-- 6. Haskell epilogue: data type and function definitions

{
data Token
  = Number Integer
  | Plus
  | Minus
  | Times
  | Div
  deriving (Eq, Ord, Show)
}
