
-- Input file to happy to generate parser for arithmetical expressions.

-- A happy file consists of 6 parts.
-- Haskell code is contained in { braces }.

-- 1. Haskell prologue: module header, import statements.

{
module CalcParser where

import Exp
import CalcLexer
}

-- 2. Basic declarations

-- The name of our parsing function.

%name calcParser

-- The haskell code to invoke for parse errors.
-- The function parseError is defined below.

%error { parseError }

-- 3. Tokens we use in grammar

%tokentype { Token  }
%token
  '+'      { Plus    }
  '-'      { Minus   }
  '*'      { Times   }
  '/'      { Div     }
  num      { Number $$ }  -- $$ holds the value of the token

-- 4. Magic separator

%%

-- 5. Productions of our grammar

-- The first non-terminal is what is parsed by "calcParser".
-- The attached haskell code computes the value of this production.
-- $n refers to the value of the nth item.

Exp
  : num               { EInt  $1 }
  | Exp '+' Exp       { EPlus  $1 $3 }
  | Exp '-' Exp       { EMinus $1 $3 }
  | Exp '*' Exp       { ETimes $1 $3 }
  | Exp '/' Exp       { EDiv   $1 $3 }

-- 6. Haskell epilogue: any Haskell code.

{
parseError :: [Token] -> a
parseError toks = error ("Parse error at " ++ show (take 5 toks))
}
